//
//  PhotosDAO.m
//  MyDBProject
//
//  Created by Blake Ward on 9/13/10.
//  Copyright (c) 2010 __MyCompanyName__. All rights reserved.
//

#import "PhotosDAO.h"
#import "PhotoDAO.h"

@implementation PhotosDAO

+ (void) moveDatabase { 
    NSFileManager *fileManager = [NSFileManager defaultManager]; 
    NSString *theDBPath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"myDatabase.sqlite"]; 
    NSError *error; 
    BOOL success; 
    NSArray * paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES); 
    NSString * docsDir = [paths objectAtIndex:0]; 
    NSString * newPath = [docsDir stringByAppendingPathComponent:@"myDatabase.sqlite"]; 
    [fileManager removeItemAtPath:newPath error: &error]; 
    success = [fileManager copyItemAtPath:theDBPath toPath:newPath error: &error];  
    if (!success) {  
        NSLog(@"Failed to copy database...error handling here %@.", [error localizedDescription]);
    }
}

- (NSMutableArray *) getAllPhotos {
    NSMutableArray * photosArray = [[NSMutableArray alloc] init]; 
    @try { 
        NSFileManager *fileManager = [NSFileManager defaultManager]; 
        NSArray * paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES); 
        NSString * docsDir = [paths objectAtIndex:0];
        NSString * theDBPath = [docsDir stringByAppendingPathComponent: @"myDatabase.sqlite"]; 
        BOOL success = [fileManager fileExistsAtPath:theDBPath]; 
        if (!success) { 
            NSLog(@"Failed to find database file '%@'."); 
        } 
        if (!(sqlite3_open([theDBPath UTF8String], &database) == SQLITE_OK)) { 
            NSLog(@"An error opening database, normally handle error here.");  
        } 
        const char *sql = "SELECT id,name,photo FROM photos"; 
        sqlite3_stmt *statement; 
        if (sqlite3_prepare_v2(database, sql, -1, &statement, NULL) != SQLITE_OK) {  
            NSLog(@"Error, failed to prepare statement, normally handle error here.");  
        } 
        while (sqlite3_step(statement) == SQLITE_ROW) { 
            PhotoDAO * aPhoto = [[PhotoDAO alloc] init]; 
            aPhoto.photoID = sqlite3_column_int(statement, 0); 
            aPhoto.name = [NSString stringWithUTF8String:(char *)sqlite3_column_text(statement, 1)]; 
            const char * rawData = sqlite3_column_blob(statement, 2); 
            int rawDataLength = sqlite3_column_bytes(statement, 2); 
            NSData *data = [NSData dataWithBytes:rawData length: rawDataLength]; 
            aPhoto.photo = [[UIImage alloc] initWithData:data]; 
            [photosArray addObject:aPhoto]; 
        } 
        if(sqlite3_finalize(statement) != SQLITE_OK) {
            NSLog(@"Failed to finalize data statement, normally error handling here.");
        } 
        if (sqlite3_close(database) != SQLITE_OK) { 
            NSLog(@"Failed to close database, normally error handling here.");
        } 
    } 
    @catch (NSException *e) { 
        NSLog(@"An exception occurred: %@", [e reason]); 
        return nil;  
    } 
    return photosArray;
}

- (void) addPhoto : (PhotoDAO *) photo { 
    const char * sql = "insert into photos (name, photo) values (?, ?)"; 
    sqlite3_stmt *insert_statement = nil; 
    NSArray * paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString * docsDir = [paths objectAtIndex:0]; 
    NSString * thePath = [docsDir stringByAppendingPathComponent: @"myDatabase.sqlite"];
    sqlite3_open([thePath UTF8String], &database); 
    sqlite3_prepare_v2(database, sql, -1, &insert_statement, NULL); 
    sqlite3_bind_text(insert_statement, 1, [photo.name UTF8String], -1, SQLITE_TRANSIENT);
    NSData * binData = UIImagePNGRepresentation(photo.photo); 
    sqlite3_bind_blob(insert_statement, 2, [binData bytes], [binData length], SQLITE_TRANSIENT);
    sqlite3_step(insert_statement); 
    sqlite3_finalize(insert_statement); 
    sqlite3_close(database);
}

- (void) changeAPhotoImage: (UIImage *) image theID: (NSInteger) photoID { 
    const char * sql = "update photos set photo = ? where id = ?"; 
    sqlite3_stmt *update_statement = nil; 
    NSArray * paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES); 
    NSString * docsDir = [paths objectAtIndex:0]; 
    NSString * thePath = [docsDir  stringByAppendingPathComponent:@"myDatabase.sqlite"]; 
    sqlite3_open([thePath UTF8String], &database); 
    sqlite3_prepare_v2(database, sql, -1, &update_statement, NULL); 
    NSData * binData = UIImagePNGRepresentation(image); 
    sqlite3_bind_blob(update_statement, 1, [binData bytes], [binData length], SQLITE_TRANSIENT); 
    sqlite3_bind_int(update_statement, 2, photoID); 
    sqlite3_step(update_statement); 
    sqlite3_finalize(update_statement); 
    sqlite3_close(database);
}

- (void) deletePhoto: (NSInteger) photoID { 
    const char * sql = "delete from photos where id = ?"; 
    sqlite3_stmt *delete_statement = nil; 
    NSArray * paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString * docsDir = [paths objectAtIndex:0]; 
    
    NSString * thePath = [docsDir stringByAppendingPathComponent: @"myDatabase.sqlite"];
    sqlite3_open([thePath UTF8String], &database); 
    
    sqlite3_prepare_v2(database, sql, -1, &delete_statement, NULL); 
    sqlite3_bind_int(delete_statement, 1, photoID); 
    sqlite3_step(delete_statement); 
    sqlite3_finalize(delete_statement); 
    sqlite3_close(database);
}

@end
