//
//  BreedViewController.m
//  DogBreeds
//
//  Created by Blake Ward on 9/16/10.
//  Copyright (c) 2010 __MyCompanyName__. All rights reserved.
//

#import "BreedViewController.h"
#import "Breed.h"
#import "AKCGroup.h"

@implementation BreedViewController

@synthesize nameField;
@synthesize breedDescriptionField;
@synthesize photoURLField;
@synthesize breed;

@synthesize insertingBreed;
@synthesize selectedGroup;

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    self.nameField.text = self.breed.name;
    self.breedDescriptionField.text = self.breed.breedDescription;
    self.photoURLField.text = self.breed.photoURL;
}

/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        // Custom initialization
    }
    return self;
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
    UIBarButtonItem *doneButton = [[UIBarButtonItem alloc]
                                   initWithBarButtonSystemItem:UIBarButtonSystemItemDone
                                   target:self action:@selector(done)];
    self.navigationItem.rightBarButtonItem = doneButton;
    [doneButton release];
    
    UIBarButtonItem *cancelButton = [[UIBarButtonItem alloc]
                                     initWithBarButtonSystemItem:UIBarButtonSystemItemCancel
                                     target:self action:@selector(cancel)];
    self.navigationItem.leftBarButtonItem = cancelButton;
    [cancelButton release];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)orientation {
	if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
		return YES;
	return NO;
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)done {
    self.breed.name = nameField.text;
    [self.nameField resignFirstResponder];
    self.breed.breedDescription = breedDescriptionField.text;
    [self.breedDescriptionField resignFirstResponder];
    self.breed.photoURL = photoURLField.text;
    [self.photoURLField resignFirstResponder];

    NSError *error = nil;
    if (![self.breed.managedObjectContext save:&error]) {
		// Be sure to handle any errors
    }
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)cancel {
    [self.nameField resignFirstResponder];
    [self.breedDescriptionField resignFirstResponder];
    [self.navigationController popViewControllerAnimated:YES];
    // If this was a new breed that was created, then cancel should get rid of the empty breed from the database
    if (insertingBreed == YES) {
        [self.selectedGroup removeBreedsObject:self.breed];
        // Delete the managed object for the given index path
        NSManagedObjectContext *context = self.breed.managedObjectContext;
        [context deleteObject:self.breed];
        
        // Save the deletion
        NSError *error = nil;
        if (![context save:&error]) {
            NSLog(@"Failed to save to data store: %@", [error localizedDescription]);
            NSArray* detailedErrors = [[error userInfo] objectForKey:NSDetailedErrorsKey];
            if(detailedErrors != nil && [detailedErrors count] > 0) {
                for(NSError* detailedError in detailedErrors) {
                    NSLog(@"  DetailedError: %@", [detailedError userInfo]);
                }
            }
            else {
                NSLog(@"  %@", [error userInfo]);
            }
            abort();
        }
    }
}


- (void)dealloc {
    [nameField release];
    [breedDescriptionField release];
    [photoURLField release];
    [breed release];
    [selectedGroup release];
    [super dealloc];
}


@end
